'----------------------------------------------------------------
'Ansteuerungsbeispiel fr Feuchtesensor SHT15 (#1515-0)
'CCTools 2005 cctools.hs-control.de (19.04.2005)
'----------------------------------------------------------------
'Modifiziert fr C-Control 1 M-Unit 2.0 mit 2x16 LCD-Display
'VD-SoftWare 2005 V-NetWork.de (26.09.2005)
'----------------------------------------------------------------
'Der rH-Wert ist nicht Temperaturkompensiert !
' Nachtrag zur Temperaturkompensation: (by Andr Helbig)
' Die C-Control I hat hierfr zu wenig Ressourcen.
'  Die Kompensation msste ber folgende Formel laufen:
'   RHlin=((405*SOrh - 40000)*250 + ((-7)*SOrh*SOrh))/25 ' Linearisierung, wird bei der CC1 mit SHT.tab gemacht
'   hyg=(((temp-2500)*(1000+8*SOrh)/100)+RHlin)/1000     ' Temperaturkompensation, es wird ein LONG-Integer-Bereich bentigt !
'  Daher bleibt die Temperaturkompensation nur der C-Control II, sowie der C-Control Pro vorbehalten.
'  Evtl. wre auch die Open-Micro/-Mini dazu in der Lage, da man mit dieser durch
'  ein paar extra Features auch 24Bit und 32Bit Zahlen bedingt verwenden kann.

  DEFINE Clock   PORT[7]
  DEFINE Data    PORT[8]

  DEFINE TW    WORD[1]   'Temporres Word
  DEFINE TH    BYTE[1]   'Temporres Hi-Byte
  DEFINE TL    BYTE[2]   'Temporres Lo-Byte

  DEFINE ACK   BYTE[3]
  DEFINE Z     BYTE[4]

  DEFINE DW    WORD[3]   'Daten Word
  DEFINE DH    BYTE[5]   'Daten Hi-Byte
  DEFINE DL    BYTE[6]   'Daten Lo-Byte

  DEFINE T     WORD[4]   'Temperatur
  DEFINE H     WORD[5]   'Feuchte

  DEFINE TX    WORD[6]   'Temporres Word

  DEFINE S     BYTE[24]  'Zwischenspeicherung Sekunde


'--------------------------------------------------------------------------------
#START

  PRINT "#ON_CONFIG#";          'Erweiterte Funktion der M2: System-Konfiguration.
  PUT &B00001000                'Interne PullUp-Widerstaende (1-8) des CC einschalten.
  PRINT "#OFF#";                'Schliesse Konfiguration.

  PRINT "#ON_LCD#";             'Erweiterte Funktion der M2: LCD-Display Ansteuerung.
  PRINT "#INIT#";               'Display initialisieren.
  PRINT "#CLR#";                'Display loeschen.
  PRINT "#L101#";               'Cursor in Zeile 1, Spalte 1 setzen.

  GOSUB SHTinit                 'Sensor Initialisieren


'--------------------------------------------------------------------------------
#LOOP                           'Hauptschleife

  IF S = SECOND THEN GOTO LOOP  'Schleife solange Sekunde sich nicht aendert.
  S = SECOND                    'Sensor sollte nicht zu oft abgefragt werden
                                'wegen Selbsterwrmung. Alternativ kann auch
                                'PAUSE 50 benutzt werden fr 1 Sek. Pause.

  PRINT "#L201#";: GOSUB ZEIT   'Unterprogramm Zeit-Ausgabe
  PRINT "#L101#";: GOSUB TEMP   '              Temperatur-Ausgabe
  PRINT "#L110#";: GOSUB HYGRO  '              Feuchte-Ausgabe

  GOTO LOOP


'--------------------------------------------------------------------------------
#TEMP  'Temperatur-Ausgabe auf LCD-Display:

  GOSUB SHTgetT: T = DW                'Hole (T) werte vom Sensor.
  IF (T < -4000) THEN GOTO TEMPerror   'Wenn -4000, dann Sensor-Fehler!

  'Optische aufbereitung fuer LCD. Kann auch weggelassen werden!
  IF (T < 1000) AND (T > 0) THEN PRINT " ";
  IF (T > -1000) AND (T < 0) THEN PRINT " ";
  IF (T >= 0) THEN PRINT "+";
  IF (T < 0) AND (T > -100) THEN PRINT "-";

  'Ausgabe des Temperatur-Wertes ( =  auf LCD mit jap. Zeichensatz)
  PRINT (T / 100); ","; ABS((T / 10) MOD 10); ABS(T MOD 10); "C";

  RETURN

#TEMPerror

  PRINT "Fehler! ";

  RETURN


'--------------------------------------------------------------------------------
#HYGRO  'Feuchte-Ausgabe auf LCD-Display:

  GOSUB SHTgetH                 'Hole Wert (H) vom Sensor

  'Leerzeichen, falls Vorkomma-Wert < 10 (optische Gruende)
  IF (H < 100) AND (H >= 0) THEN PRINT " ";

  'Ausgabe des Feuchte-Wertes
  IF H >= 0 THEN PRINT (H / 10); ","; (H MOD 10); "%rH"; ELSE PRINT "Fehler!";

  RETURN


'--------------------------------------------------------------------------------
#SHTgetT  'Temperatur-Wert des Sensors abfragen.

  GOSUB SHTstart
  TH = &b00011: GOSUB SHTwrite
  GOSUB SHTreadInt: DW = DW - 4000

  RETURN  'DW


'--------------------------------------------------------------------------------
#SHTgetH  'Feuchtigkeits-Wert des Sensors abfragen.

  GOSUB SHTstart

  TH = &b00101: GOSUB SHTwrite
  GOSUB SHTreadInt: H = DW

  IF (H AND &h8000) <> 0 THEN H = -1
  IF H = -1 THEN RETURN

  TW = H SHR 4
  Z = H MOD 16
  LOOKTAB SHTAB, TW, TX
  TW = TW - 1
  LOOKTAB SHTAB, TW, TW
  TX = ((TX - TW) * Z) SHR 4
  H = TW + TX


  RETURN  'H


'--------------------------------------------------------------------------------
#SHTinit  'Initialisiere

  DEACT data
  Clock = 0

  FOR Z = 0 TO 10
    PULSE Clock
  NEXT Z

  RETURN


'--------------------------------------------------------------------------------
#SHTstart  'Start-Sequenz

  Clock = -1
  Data  = 0
  Clock = 0
  Clock = -1
  DEACT Data
  Clock = 0

  RETURN


'--------------------------------------------------------------------------------
#SHTwrite  'Sende Daten an Sensor

  FOR Z = 0 TO 7
    IF ((TH SHL Z) AND &h80) <> 0 THEN DEACT Data ELSE Data = 0
    PULSE Clock
  NEXT Z

  DEACT Data
  Clock = -1
  TH = (Data = 0)
  Clock = 0

  RETURN  'TH

'--------------------------------------------------------------------------------
#SHTreadInt

  TW = TIMER

  DW = &HF000
  #SHTloop
    IF (TIMER - TW) >= 50 THEN GOTO SHTloopZ
  IF Data <> 0 THEN GOTO SHTloop

  ACK = 1
  GOSUB SHTread: DH = TH

  ACK = 0
  GOSUB SHTread: DL = TH

  #SHTloopZ

  RETURN

'--------------------------------------------------------------------------------
#SHTread

  TH = 0

  FOR Z = 0 TO 7
    Clock = -1
    TH = ((TH SHL 1) OR (Data AND 1))
    Clock = 0
  NEXT Z

  IF ACK <> 0 THEN Data = 0 ELSE DEACT Data

  PULSE Clock
  DEACT Data

  RETURN  'TH

'--------------------------------------------------------------------------------
#ZEIT                           'Zeit-Ausgabe

  IF HOUR   < 10 THEN PRINT "0"; : PRINT HOUR; ":";
  IF MINUTE < 10 THEN PRINT "0"; : PRINT MINUTE; ":";
  IF SECOND < 10 THEN PRINT "0"; : PRINT SECOND;

  RETURN

'--------------------------------------------------------------------------------

  TABLE SHTAB "SHT.TAB"


